package watchdog;

public interface Disparable 
{
	public enum Continuidad {ONESHOT, CONTINUO};
	
	/**
	 * Arma el watchdog con el callback de monitorizado
	 * 
	 * Para cambiar los parametros hace falta desarmarlo para volverlo a armar
	 * 
	 * @param monitorizado:  Objeto que implementa el callback
	 * @param timeout:       Tiempo de espera del watchdog      
	 * @param continuo:      True, el watchdog sigue funcionando y armado despues de dispararse
	 *                       False: el watchdog solo se dispara una vez.  Luego hay que rearmarlo
	 * 
	 * @return               True:  Exito, False: No se pudo ejecutar la peticion 
	 *                              (el watchdog sigue armado)
	 */
	public boolean arma(Monitorizable monitorizado, long timeout, Continuidad continuidad);

	/**
	 * Segnaliza al watchdog que el operario sigue vivo.
	 * 
	 * Si el watchdog esta armado, se restablece el tiempo de espera.
	 * Si el watchdog esta desarmado esta llamada se ignora
	 */
	public void i_m_alive();
	
	/**
	 * Desarma el watchdog eliminando el disparo pendiente y los futuros caso de ser
	 * periodico.
	 * 
	 * A partir de este momento el watchdog se puede volver a armar de nuevo con otros
	 * parametros.
	 * 
	 * Si el watchdog ya estaba desarmado esta llamada se ignora.
	 */
	public void desarma();
	
	/**
	 * Termina el watchdog y libera sus recursos (terminando el sub-thread)
	 * Los disparos pendientes tambien se anulan
	 * 
	 * A partir de esta llamada no se puede volver a armar el watchdog
	 */
	public void termina();
}
