package watchdog;
import java.util.Timer;
import java.util.TimerTask;



public class Watchdog extends Thread implements Disparable
{
	
	private long timeout_ms;
	private Continuidad continuidad;

	private Object lock_watchdog;

	private Timer timer;
	private ExpiraTimerTask expira_timertask;
	
	private Monitorizable observado;
	
	private boolean watchdog_armado;
	
	public Watchdog()
	{
		this.timer = new Timer();
		lock_watchdog = new Object();
		watchdog_armado = false;
		
	}

	/**
	 * Activamos hasta nueva orden (desactiva)
	 */
	public boolean arma(Monitorizable controlado, long timeout_ms, Continuidad continuidad)
	{
		synchronized(lock_watchdog)
		{
		
			if (watchdog_armado)
			{
				// No armamos el watchdog sin cancelarlo antes
				return false;
			}
			watchdog_armado = true;
			this.observado = controlado;
			this.timeout_ms = timeout_ms;
			this.continuidad = continuidad;

			set_timer();

			return true;
		}
	}

	public void i_m_alive()
	{
		synchronized(lock_watchdog)
		{
			if (watchdog_armado)
			{
				expira_timertask.cancel();
				set_timer();
			}
		}
	}
	
	public void desarma()
	{
		synchronized(lock_watchdog)
		{
			watchdog_armado = false;
			expira_timertask.cancel();
		}
	}

	public void termina()
	{
		timer.cancel();
	}

	
	private void set_timer()
	{
		expira_timertask = new ExpiraTimerTask(); // Necesario porque en los timers
		// una vez el timertask esta cancelado
		// no se puede volver a planificar

		if (continuidad == Continuidad.CONTINUO)
		{
			timer.scheduleAtFixedRate(expira_timertask, timeout_ms, timeout_ms);
		}
		else if (continuidad == Continuidad.ONESHOT)
		{
			timer.schedule(expira_timertask, timeout_ms);
		}

		
	}
	
	private class ExpiraTimerTask extends TimerTask
	{
		public void run()
		{
			synchronized(lock_watchdog)
			{
				if (continuidad == Continuidad.ONESHOT)
				{
					watchdog_armado = false;
				}
				observado.accion_recuperacion();
			}
	
		}
	}

}
